@@ -173,6 +173,17 @@ int             key_fire = KEY_RCTRL;
 int		key_use = ' ';
 int		key_strafe = KEY_RALT;
 int		key_speed = KEY_RSHIFT; 
+
+int             key_weapon1 = '1';
+int             key_weapon2 = '2';
+int             key_weapon3 = '3';
+int             key_weapon4 = '4';
+int             key_weapon5 = '5';
+int             key_weapon6 = '6';
+int             key_weapon7 = '7';
+int             key_weapon8 = '8';
+
+int             key_pause = KEY_PAUSE;
  
 int             mousebfire = 0;
 int             mousebstrafe = 1;
@@ -211,6 +222,17 @@ fixed_t         forwardmove[2] = {0x19, 0x32};
 fixed_t         sidemove[2] = {0x18, 0x28}; 
 fixed_t         angleturn[3] = {640, 1280, 320};    // + slow turn 
 
+static int *weapon_keys[] = {
+    &key_weapon1,
+    &key_weapon2,
+    &key_weapon3,
+    &key_weapon4,
+    &key_weapon5,
+    &key_weapon6,
+    &key_weapon7,
+    &key_weapon8
+};
+
 #define SLOWTURNTICS	6 
  
 #define NUMKEYS		256 
@@ -501,13 +523,18 @@ void G_BuildTiccmd (ticcmd_t* cmd)
     } 
 
     // chainsaw overrides 
-    for (i=0 ; i<NUMWEAPONS-1 ; i++)        
-	if (gamekeydown['1'+i]) 
-	{ 
+
+    for (i=0; i<arrlen(weapon_keys); ++i)
+    {
+        int key = *weapon_keys[i];
+
+        if (gamekeydown[key])
+        {
 	    cmd->buttons |= BT_CHANGE; 
 	    cmd->buttons |= i<<BT_WEAPONSHIFT; 
 	    break; 
-	}
+        }
+    }
     
     // mouse
     if (mousebuttons[mousebforward]) 
@@ -623,10 +650,20 @@ void G_BuildTiccmd (ticcmd_t* cmd)
 
     if (lowres_turn)
     {
-        // round angleturn to the nearest 256 boundary
+        static signed short carry = 0;
+        signed short desired_angleturn;
+
+        desired_angleturn = cmd->angleturn + carry;
+
+        // round angleturn to the nearest 256 unit boundary
         // for recording demos with single byte values for turn
 
-        cmd->angleturn = (cmd->angleturn + 128) & 0xff00;
+        cmd->angleturn = (desired_angleturn + 128) & 0xff00;
+
+        // Carry forward the error from the reduced resolution to the
+        // next tic, so that successive small movements can accumulate.
+
+        carry = desired_angleturn - cmd->angleturn;
     }
 } 
  
@@ -765,13 +802,15 @@ boolean G_Responder (event_t* ev)
     switch (ev->type) 
     { 
       case ev_keydown: 
-	if (ev->data1 == KEY_PAUSE) 
+	if (ev->data1 == key_pause) 
 	{ 
 	    sendpause = true; 
-	    return true; 
-	} 
-	if (ev->data1 <NUMKEYS) 
+	}
+        else if (ev->data1 <NUMKEYS) 
+        {
 	    gamekeydown[ev->data1] = true; 
+        }
+
 	return true;    // eat key down events 
  
       case ev_keyup: 
