@@ -50,6 +50,9 @@
 #include "w_wad.h"
 #include "z_zone.h"
 
+#define LOADING_DISK_W 16
+#define LOADING_DISK_H 16
+
 // Non aspect ratio-corrected modes (direct multiples of 320x200)
 
 static screen_mode_t *screen_modes[] = {
@@ -98,7 +101,7 @@ static int windowwidth, windowheight;
 
 // display has been set up?
 
-static boolean initialised = false;
+static boolean initialized = false;
 
 // disable mouse?
 
@@ -145,7 +148,6 @@ boolean screenvisible;
 // restored by EndRead
 
 static byte *disk_image = NULL;
-static int disk_image_w, disk_image_h;
 static byte *saved_background;
 static boolean window_focused;
 
@@ -182,7 +184,7 @@ static boolean MouseShouldBeGrabbed()
     if (screensaver_mode)
         return false;
 
-    // if the window doesnt have focus, never grab it
+    // if the window doesn't have focus, never grab it
 
     if (!window_focused)
         return false;
@@ -193,6 +195,17 @@ static boolean MouseShouldBeGrabbed()
     if (fullscreen)
         return true;
 
+#ifdef _WIN32_WCE
+
+    // On Windows CE, always grab input.  This is because hardware
+    // button events are only acquired by SDL when the input is grabbed.
+    // Almost all Windows CE devices should have touch screens anyway,
+    // so this shouldn't affect mouse grabbing behavior.
+
+    return true;
+
+#else
+
     // Don't grab the mouse if mouse input is disabled
 
     if (!usemouse || nomouse)
@@ -204,18 +217,20 @@ static boolean MouseShouldBeGrabbed()
         return false;
 
     // if we specify not to grab the mouse, never grab
- 
+
     if (!grabmouse)
         return false;
 
     // when menu is active or game is paused, release the mouse 
- 
+
     if (menuactive || paused)
         return false;
 
     // only grab mouse when playing levels (but not demos)
 
     return (gamestate == GS_LEVEL) && !demoplayback;
+
+#endif /* #ifndef _WIN32_WCE */
 }
 
 // Update the value of window_focused when we get a focus event
@@ -231,7 +246,7 @@ static void UpdateFocus(void)
     state = SDL_GetAppState();
 
     // We should have input (keyboard) focus and be visible 
-    // (not minimised)
+    // (not minimized)
 
     window_focused = (state & SDL_APPINPUTFOCUS) && (state & SDL_APPACTIVE);
 
@@ -245,6 +260,8 @@ static void LoadDiskImage(void)
     patch_t *disk;
     char *disk_name;
     int y;
+    int xoffset = SCREENWIDTH - LOADING_DISK_W;
+    int yoffset = SCREENHEIGHT - LOADING_DISK_H;
     char buf[20];
 
     SDL_VideoDriverName(buf, 15);
@@ -265,19 +282,20 @@ static void LoadDiskImage(void)
 
     disk = W_CacheLumpName(disk_name, PU_STATIC);
 
-    V_DrawPatch(0, 0, 0, disk);
-    disk_image_w = SHORT(disk->width);
-    disk_image_h = SHORT(disk->height);
+    // Draw the disk to the screen:
 
-    disk_image = Z_Malloc(disk_image_w * disk_image_h, PU_STATIC, NULL);
-    saved_background = Z_Malloc(disk_image_w * disk_image_h, PU_STATIC, NULL);
+    V_DrawPatch(SCREENWIDTH - LOADING_DISK_W,
+                SCREENHEIGHT - LOADING_DISK_H,
+                0, disk);
 
-    for (y=0; y<disk_image_h; ++y) 
+    disk_image = Z_Malloc(LOADING_DISK_W * LOADING_DISK_H, PU_STATIC, NULL);
+    saved_background = Z_Malloc(LOADING_DISK_W * LOADING_DISK_H, PU_STATIC, NULL);
+
+    for (y=0; y<LOADING_DISK_H; ++y) 
     {
-        memcpy(disk_image + disk_image_w * y,
-               screens[0] + SCREENWIDTH * y,
-               disk_image_w);
-        memset(screens[0] + SCREENWIDTH * y, 0, disk_image_w);
+        memcpy(disk_image + LOADING_DISK_W * y,
+               screens[0] + SCREENWIDTH * (y + yoffset) + xoffset,
+               LOADING_DISK_W);
     }
 
     W_ReleaseLumpName(disk_name);
@@ -316,7 +334,9 @@ static int TranslateKey(SDL_keysym *sym)
 
       case SDLK_PAUSE:	return KEY_PAUSE;
 
+#if !SDL_VERSION_ATLEAST(1, 3, 0)
       case SDLK_EQUALS: return KEY_EQUALS;
+#endif
 
       case SDLK_MINUS:          return KEY_MINUS;
 
@@ -329,9 +349,11 @@ static int TranslateKey(SDL_keysym *sym)
 	return KEY_RCTRL;
 	
       case SDLK_LALT:
-      case SDLK_LMETA:
       case SDLK_RALT:
+#if !SDL_VERSION_ATLEAST(1, 3, 0)
+      case SDLK_LMETA:
       case SDLK_RMETA:
+#endif
         return KEY_RALT;
 
       case SDLK_CAPSLOCK: return KEY_CAPSLOCK;
@@ -362,6 +384,15 @@ static int TranslateKey(SDL_keysym *sym)
       case SDLK_PAGEUP: return KEY_PGUP;
       case SDLK_PAGEDOWN: return KEY_PGDN;
 
+#ifdef SDL_HAVE_APP_KEYS
+        case SDLK_APP1:        return KEY_F1;
+        case SDLK_APP2:        return KEY_F2;
+        case SDLK_APP3:        return KEY_F3;
+        case SDLK_APP4:        return KEY_F4;
+        case SDLK_APP5:        return KEY_F5;
+        case SDLK_APP6:        return KEY_F6;
+#endif
+
       default:
         return tolower(sym->sym);
     }
@@ -369,14 +400,15 @@ static int TranslateKey(SDL_keysym *sym)
 
 void I_ShutdownGraphics(void)
 {
-    if (initialised)
+    if (initialized)
     {
+        SDL_SetCursor(cursors[1]);
         SDL_ShowCursor(1);
         SDL_WM_GrabInput(SDL_GRAB_OFF);
 
         SDL_QuitSubSystem(SDL_INIT_VIDEO);
     
-        initialised = false;
+        initialized = false;
     }
 }
 
@@ -393,9 +425,15 @@ void I_StartFrame (void)
 
 static int MouseButtonState(void)
 {
-    Uint8 state = SDL_GetMouseState(NULL, NULL);
+    Uint8 state;
     int result = 0;
 
+#if SDL_VERSION_ATLEAST(1, 3, 0)
+    state = SDL_GetMouseState(0, NULL, NULL);
+#else
+    state = SDL_GetMouseState(NULL, NULL);
+#endif
+
     // Note: button "0" is left, button "1" is right,
     // button "2" is middle for Doom.  This is different
     // to how SDL sees things.
@@ -480,13 +518,20 @@ void I_GetEvent(void)
                     event.data2 = sdlevent.key.keysym.unicode;
                 }
 
-                D_PostEvent(&event);
+                if (event.data1 != 0)
+                {
+                    D_PostEvent(&event);
+                }
                 break;
 
             case SDL_KEYUP:
                 event.type = ev_keyup;
                 event.data1 = TranslateKey(&sdlevent.key.keysym);
-                D_PostEvent(&event);
+
+                if (event.data1 != 0)
+                {
+                    D_PostEvent(&event);
+                }
                 break;
 
                 /*
@@ -551,7 +596,11 @@ static void CenterMouse(void)
     // Clear any relative movement caused by warping
 
     SDL_PumpEvents();
+#if SDL_VERSION_ATLEAST(1, 3, 0)
+    SDL_GetRelativeMouseState(0, NULL, NULL);
+#else
     SDL_GetRelativeMouseState(NULL, NULL);
+#endif
 }
 
 //
@@ -565,7 +614,11 @@ static void I_ReadMouse(void)
     int x, y;
     event_t ev;
 
+#if SDL_VERSION_ATLEAST(1, 3, 0)
+    SDL_GetRelativeMouseState(0, &x, &y);
+#else
     SDL_GetRelativeMouseState(&x, &y);
+#endif
 
     if (x != 0 || y != 0) 
     {
@@ -588,7 +641,7 @@ static void I_ReadMouse(void)
 //
 void I_StartTic (void)
 {
-    if (!initialised)
+    if (!initialized)
     {
         return;
     }
@@ -702,50 +755,54 @@ static void UpdateRect(int x1, int y1, int x2, int y2)
 
 void I_BeginRead(void)
 {
+    byte *screenloc = screens[0]
+                    + (SCREENHEIGHT - LOADING_DISK_H) * SCREENWIDTH
+                    + (SCREENWIDTH - LOADING_DISK_W);
     int y;
 
-    if (!initialised || disk_image == NULL)
+    if (!initialized || disk_image == NULL)
         return;
 
     // save background and copy the disk image in
 
-    for (y=0; y<disk_image_h; ++y)
+    for (y=0; y<LOADING_DISK_H; ++y)
     {
-        byte *screenloc = 
-               screens[0] 
-                 + (SCREENHEIGHT - 1 - disk_image_h + y) * SCREENWIDTH
-                 + (SCREENWIDTH - 1 - disk_image_w);
-
-        memcpy(saved_background + y * disk_image_w,
+        memcpy(saved_background + y * LOADING_DISK_W,
                screenloc,
-               disk_image_w);
-        memcpy(screenloc, disk_image + y * disk_image_w, disk_image_w);
+               LOADING_DISK_W);
+        memcpy(screenloc,
+               disk_image + y * LOADING_DISK_W,
+               LOADING_DISK_W);
+
+        screenloc += SCREENWIDTH;
     }
 
-    UpdateRect(SCREENWIDTH - disk_image_w, SCREENHEIGHT - disk_image_h,
+    UpdateRect(SCREENWIDTH - LOADING_DISK_W, SCREENHEIGHT - LOADING_DISK_H,
                SCREENWIDTH, SCREENHEIGHT);
 }
 
 void I_EndRead(void)
 {
+    byte *screenloc = screens[0]
+                    + (SCREENHEIGHT - LOADING_DISK_H) * SCREENWIDTH
+                    + (SCREENWIDTH - LOADING_DISK_W);
     int y;
 
-    if (!initialised || disk_image == NULL)
+    if (!initialized || disk_image == NULL)
         return;
 
     // save background and copy the disk image in
 
-    for (y=0; y<disk_image_h; ++y)
+    for (y=0; y<LOADING_DISK_H; ++y)
     {
-        byte *screenloc = 
-               screens[0] 
-                 + (SCREENHEIGHT - 1 - disk_image_h + y) * SCREENWIDTH
-                 + (SCREENWIDTH - 1 - disk_image_w);
+        memcpy(screenloc,
+               saved_background + y * LOADING_DISK_W,
+               LOADING_DISK_W);
 
-        memcpy(screenloc, saved_background + y * disk_image_w, disk_image_w);
+        screenloc += SCREENWIDTH;
     }
 
-    UpdateRect(SCREENWIDTH - disk_image_w, SCREENHEIGHT - disk_image_h,
+    UpdateRect(SCREENWIDTH - LOADING_DISK_W, SCREENHEIGHT - LOADING_DISK_H,
                SCREENWIDTH, SCREENHEIGHT);
 }
 
@@ -759,7 +816,7 @@ void I_FinishUpdate (void)
     int		i;
     // UNUSED static unsigned char *bigscreen=0;
 
-    if (!initialised)
+    if (!initialized)
         return;
 
     if (noblit)
@@ -964,125 +1021,155 @@ static screen_mode_t *I_FindScreenMode(int w, int h)
     return best_mode;
 }
 
-// If the video mode set in the configuration file is not available,
-// try to choose a different mode.
+// Adjust to an appropriate fullscreen mode.
+// Returns true if successful.
 
-static void I_AutoAdjustSettings(void)
+static boolean AutoAdjustFullscreen(void)
 {
-    if (fullscreen)
+    SDL_Rect **modes;
+    SDL_Rect *best_mode;
+    screen_mode_t *screen_mode;
+    int target_pixels, diff, best_diff;
+    int i;
+
+    modes = SDL_ListModes(NULL, SDL_FULLSCREEN);
+
+    // No fullscreen modes available at all?
+
+    if (modes == NULL || modes == (SDL_Rect **) -1 || *modes == NULL)
     {
-        SDL_Rect **modes;
-        SDL_Rect *best_mode;
-        screen_mode_t *screen_mode;
-        int target_pixels, diff, best_diff;
-        int i;
-
-        modes = SDL_ListModes(NULL, SDL_FULLSCREEN);
-
-        // Find the best mode that matches the mode specified in the
-        // configuration file
-
-        best_mode = NULL;
-        best_diff = INT_MAX;
-        target_pixels = screen_width * screen_height;
-
-        for (i=0; modes[i] != NULL; ++i) 
-        {
-            //printf("%ix%i?\n", modes[i]->w, modes[i]->h);
-
-            // What screen_mode_t would be used for this video mode?
-
-            screen_mode = I_FindScreenMode(modes[i]->w, modes[i]->h);
-
-            // Never choose a screen mode that we cannot run in, or
-            // is poor quality for fullscreen
-
-            if (screen_mode == NULL || screen_mode->poor_quality)
-            {
-            //    printf("\tUnsupported / poor quality\n");
-                continue;
-            }
-
-            // Do we have the exact mode?
-            // If so, no autoadjust needed
-
-            if (screen_width == modes[i]->w && screen_height == modes[i]->h)
-            {
-            //    printf("\tExact mode!\n");
-                return;
-            }
-
-            // Is this mode better than the current mode?
-
-            diff = (screen_width - modes[i]->w) 
-                     * (screen_width - modes[i]->w)
-                 + (screen_height - modes[i]->h)
-                     * (screen_height - modes[i]->h);
-
-            if (diff < best_diff)
-            {
-            //    printf("\tA valid mode\n");
-                best_mode = modes[i];
-                best_diff = diff;
-            }
-        }
-
-        if (best_mode == NULL)
-        {
-            // Unable to find a valid mode!
-
-            I_Error("Unable to find any valid video mode at all!");
-        }
-
-        printf("I_InitGraphics: %ix%i mode not supported on this machine.\n",
-               screen_width, screen_height);
-
-        screen_width = best_mode->w;
-        screen_height = best_mode->h;
-
+        return false;
     }
-    else
+
+    // Find the best mode that matches the mode specified in the
+    // configuration file
+
+    best_mode = NULL;
+    best_diff = INT_MAX;
+    target_pixels = screen_width * screen_height;
+
+    for (i=0; modes[i] != NULL; ++i) 
     {
-        screen_mode_t *best_mode;
+        //printf("%ix%i?\n", modes[i]->w, modes[i]->h);
 
-        //
-        // Windowed mode.
-        //
-        // Find a screen_mode_t to fit within the current settings
-        //
+        // What screen_mode_t would be used for this video mode?
 
-        best_mode = I_FindScreenMode(screen_width, screen_height);
+        screen_mode = I_FindScreenMode(modes[i]->w, modes[i]->h);
 
-        if (best_mode == NULL) 
+        // Never choose a screen mode that we cannot run in, or
+        // is poor quality for fullscreen
+
+        if (screen_mode == NULL || screen_mode->poor_quality)
         {
-            // Nothing fits within the current settings.
-            // Pick the closest to 320x200 possible.
-
-            best_mode = I_FindScreenMode(SCREENWIDTH, SCREENHEIGHT_4_3);
+        //    printf("\tUnsupported / poor quality\n");
+            continue;
         }
 
-        // Do we have the exact mode already?
+        // Do we have the exact mode?
+        // If so, no autoadjust needed
 
-        if (best_mode->width == screen_width 
-         && best_mode->height == screen_height)
+        if (screen_width == modes[i]->w && screen_height == modes[i]->h)
         {
-            return;
+        //    printf("\tExact mode!\n");
+            return true;
         }
 
+        // Is this mode better than the current mode?
+
+        diff = (screen_width - modes[i]->w) * (screen_width - modes[i]->w)
+             + (screen_height - modes[i]->h) * (screen_height - modes[i]->h);
+
+        if (diff < best_diff)
+        {
+        //    printf("\tA valid mode\n");
+            best_mode = modes[i];
+            best_diff = diff;
+        }
+    }
+
+    if (best_mode == NULL)
+    {
+        // Unable to find a valid mode!
+
+        return false;
+    }
+
+    printf("I_InitGraphics: %ix%i mode not supported on this machine.\n",
+           screen_width, screen_height);
+
+    screen_width = best_mode->w;
+    screen_height = best_mode->h;
+
+    return true;
+}
+
+// Auto-adjust to a valid windowed mode.
+
+static void AutoAdjustWindowed(void)
+{
+    screen_mode_t *best_mode;
+
+    // Find a screen_mode_t to fit within the current settings
+
+    best_mode = I_FindScreenMode(screen_width, screen_height);
+
+    if (best_mode == NULL)
+    {
+        // Nothing fits within the current settings.
+        // Pick the closest to 320x200 possible.
+
+        best_mode = I_FindScreenMode(SCREENWIDTH, SCREENHEIGHT_4_3);
+    }
+
+    // Switch to the best mode if necessary.
+
+    if (best_mode->width != screen_width || best_mode->height != screen_height)
+    {
         printf("I_InitGraphics: Cannot run at specified mode: %ix%i\n",
                screen_width, screen_height);
 
         screen_width = best_mode->width;
         screen_height = best_mode->height;
     }
+}
 
-    printf("I_InitGraphics: Auto-adjusted to %ix%i.\n",
-           screen_width, screen_height);
+// If the video mode set in the configuration file is not available,
+// try to choose a different mode.
 
-    printf("NOTE: Your video settings have been adjusted.  "
-           "To disable this behavior,\n"
-           "set autoadjust_video_settings to 0 in your "
-           "configuration file.\n");
+static void I_AutoAdjustSettings(void)
+{
+    int old_screen_w, old_screen_h;
+
+    old_screen_w = screen_width;
+    old_screen_h = screen_height;
+
+    // If we are running fullscreen, try to autoadjust to a valid fullscreen
+    // mode.  If this is impossible, switch to windowed.
+
+    if (fullscreen && !AutoAdjustFullscreen())
+    {
+        fullscreen = 0;
+    }
+
+    // If we are running windowed, pick a valid window size.
+
+    if (!fullscreen)
+    {
+        AutoAdjustWindowed();
+    }
+
+    // Have the settings changed?  Show a message.
+
+    if (screen_width != old_screen_w || screen_height != old_screen_h)
+    {
+        printf("I_InitGraphics: Auto-adjusted to %ix%i.\n",
+               screen_width, screen_height);
+
+        printf("NOTE: Your video settings have been adjusted.  "
+               "To disable this behavior,\n"
+               "set autoadjust_video_settings to 0 in your "
+               "configuration file.\n");
+    }
 }
 
 // Set video size to a particular scale factor (1x, 2x, 3x, etc.)
@@ -1353,7 +1440,7 @@ static void SetSDLVideoDriver(void)
         free(env_string);
     }
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(_WIN32_WCE)
 
     // Allow -gdi as a shortcut for using the windib driver.
 
@@ -1432,7 +1519,7 @@ void I_InitGraphics(void)
 
     if (SDL_Init(SDL_INIT_VIDEO) < 0) 
     {
-        I_Error("Failed to initialise video: %s", SDL_GetError());
+        I_Error("Failed to initialize video: %s", SDL_GetError());
     }
 
     // Check for command-line video-related parameters.
@@ -1485,7 +1572,9 @@ void I_InitGraphics(void)
     // has to be done before the call to SDL_SetVideoMode.
 
     I_SetWindowCaption();
+#if !SDL_VERSION_ATLEAST(1, 3, 0)
     I_SetWindowIcon();
+#endif
 
     // Set the video mode.
 
@@ -1611,6 +1700,6 @@ void I_InitGraphics(void)
         CenterMouse();
     }
 
-    initialised = true;
+    initialized = true;
 }
 
